/*
 * @(#)file      SimpleManager.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.37
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.easymanager;

// java import
//
import java.lang.System;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.HashMap;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.awt.TextArea;

import java.awt.Frame;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.net.InetAddress;

// RI import
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpVarBind;

import com.sun.management.snmp.SnmpOidTable;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.SnmpTrapListener;
import com.sun.management.snmp.manager.SnmpInformListener;
import com.sun.management.snmp.SnmpPduRequest;
import com.sun.management.snmp.SnmpPduTrap;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpScopedPduRequest;

// jdmk import
//
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.TraceManager;

import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineId;

import com.sun.management.snmp.SnmpUnknownModelException;
import com.sun.management.snmp.SnmpEngineFactory;
import com.sun.management.snmp.SnmpEngineParameters;
import com.sun.management.internal.snmp.SnmpTools;
import com.sun.management.snmp.SnmpEventReportDispatcher;
import com.sun.jdmk.tasks.DaemonTaskServer;
import com.sun.management.snmp.SnmpOidRecord;
import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.management.snmp.SnmpOidDatabaseSupport;


class TrapFrame extends Frame
{
    public TextArea text = new TextArea(7, 40);
    public TrapFrame(String name) {
	super(name);
	text.setEditable(false);
	add(text);
	pack();
    }
}

class OidDisplay {
    private TextArea text = null;
    private SimpleManager sm = null;
    public TextArea getText() {
	return text;
    }
    
    OidDisplay(TextArea text, SimpleManager sm) {
	this.sm = sm;
	this.text = text;
    }
    
    //protected void scroll() {
    //getText().setCaretPosition(getText().getDocument().getLength()); 
    //}

    protected void displayList(SnmpVarBind[] vars) {
	for(int i = 0; i < vars.length; i++) {
	    getText().append("\tObject : " + toString(vars[i].getOid()) + "\n");
	    getText().append("\tValue : " + vars[i].getSnmpValue() + "\n");
	}
    }
    
    protected String toString(SnmpOid o) {
	String oid = o.toString();
	try {
	if(!oid.endsWith(".0")) {
	    // Table processing
	    for(Enumeration e = sm.metaFinder.entries.elements();
		e.hasMoreElements();) {
		String entry = ((SnmpOidRecord) e.nextElement()).getOid();
		if(oid.startsWith(entry)) {
		    //Compute field
		    long[] lentry = new SnmpOid(entry).longValue();
		    long[] loid = o.longValue();
		    long f = loid[lentry.length];
		    long[] fieldOid = new long[lentry.length + 1];
		    java.lang.System.arraycopy(lentry, 0, fieldOid, 0, lentry.length);
		    fieldOid[lentry.length] = f;
		    String field = new SnmpOid(fieldOid).toString();
		    // Compute index
		    int indexSize = loid.length - fieldOid.length;
		    long[] lindex = new long[indexSize];
		    for(int j= 0; j < indexSize; j++) {
			lindex[j] = loid[fieldOid.length + j];
		    }
		    
		    String index = new SnmpOid(lindex).toString();
		    
		    //text.append("INDEX : " + index);
		    
		    //text.append("FIELD : " + field);
		    
		    String tableOid = reconstructOid(entry, 1);
		    String groupOid = reconstructOid(entry, 2);
		   
		    SnmpOidRecord r = sm.resolveOidRecord(tableOid);
		    StringBuffer b = new StringBuffer();
		    if(r != null)
			b.append(r.getName());
		    else
			break;
		    b.append(".");
		    r = sm.resolveOidRecord(entry);
		    if(r != null)
			b.append(r.getName());
		    else
			break;
		    b.append("[" + index + "]");
		    b.append(".");
		    r = sm.resolveOidRecord(field);
		    if(r != null)
			b.append(r.getName());
		    else
			break;
		    
		    return b.toString();
		}
	    }
	}
	}catch(Exception e) {
	    e.printStackTrace();
	    java.lang.System.out.println("Received oid : " + oid);
	}
	return oid;
    }
    
    private String reconstructOid(String oid, int remove) {
	SnmpOid val = new SnmpOid(oid);
	long[] oidLong = val.longValue();
	SnmpOid ret = new SnmpOid();
	int size = oidLong.length - remove;
	
	for(int i = 0; i < size; i++)
	    ret.append(oidLong[i]);
	
	return ret.toString();
    }
}

class InformListenerImpl extends OidDisplay implements SnmpInformListener {
    private int count = 0;
    private synchronized int getCount() { count += 1; return count; }
    
    public void processSnmpInform(SnmpPduRequest inform) {
        
        // Display the received PDU.
        getText().append("**************INFORM***"+"["+ getCount() + "]" + "*****************\n");	
        getText().append("\tType      = " + inform.pduTypeToString(inform.type) + "\n");
        getText().append("\tVersion   = " + inform.version + "\n");
        getText().append("\tRequestId = " + inform.requestId + "\n");
        getText().append("\tAddress   = " + inform.address + "\n");
        getText().append("\tPort      = " + inform.port + "\n");
        getText().append("\tCommunity = " + new String(inform.community) + "\n");

	displayList(inform.varBindList);
	
	getText().append("**************END INFORM******************\n");
    }
    
    public void processSnmpInformV3(SnmpScopedPduRequest inform) {
	getText().append("**************V3 INFORM***"+"["+ getCount() + "]" + "*****************\n");	
	getText().append("\tContextEngineId : " + new String(inform.contextEngineId) + "\n");
	getText().append("\tContextName : " + new String(inform.contextName) + "\n");
	getText().append("\tMsgFlags : " + inform.msgFlags + "\n");
	getText().append("\tMsgMaxSize : " + inform.msgMaxSize + "\n");
	getText().append("\tMsgSecurityModel : " + inform.msgSecurityModel + "\n");
	displayList(inform.varBindList);
	

	getText().append("**************END V3 INFORM******************\n");
    }
    public InformListenerImpl(TextArea text, SimpleManager sm) {
	super(text, sm);
    }  
}

class TrapListenerImpl extends OidDisplay implements SnmpTrapListener {
    private int count = 0;
    private synchronized int getCount() { count += 1; return count; }

    public void processSnmpTrapV3(SnmpScopedPduRequest trap) {
	getText().append("**************V3 TRAP***"+"["+ getCount() + "]" + "*****************\n");	
	getText().append("\tContextEngineId : " + new String(trap.contextEngineId) + "\n");
	getText().append("\tContextName : " + new String(trap.contextName) + "\n");
	getText().append("\tMsgFlags : " + trap.msgFlags + "\n");
	getText().append("\tMsgMaxSize : " + trap.msgMaxSize + "\n");
	getText().append("\tMsgSecurityModel : " + trap.msgSecurityModel + "\n");

	displayList(trap.varBindList);

	getText().append("**************END V3 NOTIFICATION******************\n");
    }
    
    public void processSnmpTrapV1(SnmpPduTrap trap) {
	getText().append("**************V1 TRAP***"+"["+ getCount() + "]" + "*****************\n");
	getText().append("\tCommunity : " + new String(trap.community) + "\n");
	getText().append("\tEnterprise :" + toString(trap.enterprise) + "\n");
        getText().append("\tGeneric :" + trap.genericTrap + "\n");
        getText().append("\tSpecific :" + trap.specificTrap + "\n");
        getText().append("\tTimeStamp :" + trap.timeStamp + "\n");
	if(trap.agentAddr != null)
	    getText().append("\tAgent adress :"+trap.agentAddr.stringValue()+"\n");
	else
	    getText().append("\tAgent adress : null");

	displayList(trap.varBindList);

	getText().append("**************END V1 TRAP******************\n");
	//scroll();
    }
    
    public void processSnmpTrapV2(SnmpPduRequest trap) {
	SnmpVarBind[] vars = trap.varBindList;
	getText().append("**************V2 TRAP***"+"["+ getCount() + "]" + "*****************\n");
	displayList(vars);
	getText().append("**************END V2 TRAP******************\n");
	
    }
    
    public TrapListenerImpl(TextArea text, SimpleManager sm) {
	super(text, sm);
    }    
}

public class SimpleManager {
    
    private int f = 0;
    private String protocol = null;
    private Vector list = new Vector();
    String port = null;
    String host = null;
    String max = null;
    String nrp = null;
    TrapFrame trapFrame = null;
    HelpFrame helpFrame = null;
    MetaFinder metaFinder = null;
    static HashMap trapAgents = new HashMap();
    TrapListenerImpl trapListener = null;
    InformListenerImpl informListener = null;
    SnmpOidDatabaseSupport oidDatabase = null;
    boolean connected = false;
    SnmpUsmPeer v3agent = null;
    SnmpEngine engine = null;
    Hashtable profiles = new Hashtable();
    Hashtable mibFrames = new Hashtable();
    SnmpParameters v2Parameters = null;
    SnmpSession session = null;
    com.sun.management.snmp.manager.SnmpEventReportDispatcher trapAgent = null;
    private static String V1_DEFAULT_PROFILE_NAME = "JDMK_STD_V1_profile";
    private static String V1_DEFAULT_PROFILE = "1 -r public -w private";
    private static String V2_DEFAULT_PROFILE_NAME = "JDMK_STD_V2_profile";
    private static String V2_DEFAULT_PROFILE = "2 -r public -w private";
    private static String V3_DEFAULT_PROFILE_NAME = "JDMK_STD_V3_profile";
    private static String V3_DEFAULT_PROFILE = "3 -u defaultUser -f noAuthNoPriv";

    
    SnmpOidRecord resolveOidRecord(String oid) 
	throws IllegalArgumentException {
	SnmpOidRecord rec = null;
	try {
	    rec = oidDatabase.resolveVarOid(oid);
	}catch(Exception e){}
	
	return rec;
    }
    
    SnmpOid resolveOid(String oid) throws IllegalArgumentException {
	SnmpOidRecord rec = resolveOidRecord(oid);
	
	if(rec != null)
	    return new SnmpOid(rec.getOid());
	
	try {
	    rec = oidDatabase.resolveVarOid(oid);
	}catch(Exception e){}
	
	if(rec != null)
	    return new SnmpOid(rec.getOid());
	    
	return new SnmpOid(oid);
    }

    private void parse(String str) throws Exception
    {
	
	list.clear();
	while(true) {
	    StringTokenizer token = new StringTokenizer(str, ":");
	    String func = null;
	    func = token.nextToken();
	    try{
		String param = token.nextToken();
		if(param.equals("-p")) {
		    String profile = token.nextToken();
		    String p = (String) profiles.get(profile);
		    StringTokenizer t = new StringTokenizer(p);
		    protocol = t.nextToken();
		}
		else   
		    protocol = param;
	    }catch(Exception e2) { protocol= "2"; }
	    
	    if(func.equals("getnext")) f = 1;
	    if(func.equals("get")) f = 0;
	    if(func.equals("get") || func.equals("getnext")) {
		int start = str.indexOf("<");
		String sub = str.substring(start + 1);
		StringTokenizer getToken = new StringTokenizer(sub, ",");
		if(sub.equals(">")) break;
		while(getToken.hasMoreTokens()) {
		    String strOid = getToken.nextToken().trim();
		    StringTokenizer oidToken = new 
			StringTokenizer(strOid, ">");
		    try{
			strOid = oidToken.nextToken().trim();
		    }catch(Exception e2) {}
		    SnmpOid oid = resolveOid(strOid);
		    SnmpVarBind var = new SnmpVarBind(oid);
		    list.addElement(var);
		}
		break;
	    }
	    else {
		if(func.equals("set")){
		    f = 2;
		    int start = str.indexOf("<");
		    String sub = str.substring(start + 1);
		    StringTokenizer getToken = new StringTokenizer(sub, ">");
		    if(sub.equals(">")) break;
		    while(getToken.hasMoreTokens()) {
			String bind = getToken.nextToken().trim();
			int startSub = bind.indexOf("<");
			String subSet = bind.substring(startSub + 1);
			StringTokenizer setToken = 
			    new StringTokenizer(subSet, ",");
			if(subSet.equals(">")) break;
			while(setToken.hasMoreTokens()) {
			    String strOid = setToken.nextToken().trim();
			    String syntax = setToken.nextToken().trim();
			    String value = setToken.nextToken().trim();
			    StringTokenizer valToken = new 
				StringTokenizer(value, ">");
			    try{
				value = valToken.nextToken().trim();
			    }catch(Exception e2) {}
			    SnmpOid oid = resolveOid(strOid);
			    SnmpVarBind var = setSyntax(oid, syntax, value);
			    list.addElement(var);
			}
		    }
		    break;
		}
		else {
		    StringTokenizer getBToken = new StringTokenizer(str, ",");
		    String funcGetBulk = null;
		    funcGetBulk = getBToken.nextToken();
		    if(funcGetBulk.equals("getbulk")) {
			f = 3;
			nrp = getBToken.nextToken();
			StringTokenizer maxToken = 
			    new StringTokenizer(str, ":");
			String tmp = maxToken.nextToken();
			int s = tmp.lastIndexOf(",");
			max = tmp.substring(s + 1, tmp.length());
			int start = str.indexOf("<");
			String sub = str.substring(start + 1);
			if(sub.equals(">")) break;
			StringTokenizer getToken = 
			new StringTokenizer(sub, ",");
		    while(getToken.hasMoreTokens()) {
			String strOid = getToken.nextToken().trim();
			StringTokenizer oidToken = new 
			    StringTokenizer(strOid, ">");
			try{
			    strOid = oidToken.nextToken().trim();
			}catch(Exception e2) {}
			SnmpOid oid = resolveOid(strOid);
			SnmpVarBind var = new SnmpVarBind(oid);
			list.addElement(var);
		    }
		    break;
		    }
		    else 
			throw new Exception();
		}
	    }
	}
    }

    private SnmpVarBind setSyntax(SnmpOid oid,
				  String syntax,
				  String value){
	int tag = Integer.parseInt(syntax);
	switch(tag) {
	case 1: //Boolean
	    return new SnmpVarBind(oid, new SnmpInt(Boolean.getBoolean(value)));
	case 2: //Integer
	    return new SnmpVarBind(oid, new SnmpInt(Integer.parseInt(value)));
	case 4: // Octet String
	    return new SnmpVarBind(oid, new SnmpString(value));
	case 5: // Object Null
	    return new SnmpVarBind(oid, new SnmpNull());
	case 6: // Object identifier
	    return new SnmpVarBind(oid, resolveOid(value));
	case 7: { // Binary string; 
	    byte[] bin = SnmpTools.ascii2binary(value);
	    return new SnmpVarBind(oid, new SnmpString(bin));
	}
	}
	return new SnmpVarBind(oid, new SnmpNull());
    }
    
    /**
     * Called by applet only
     */
    public SimpleManager(SnmpPeer agent, 
			 SnmpSession session,
			 String host,
			 String port,
			 Iterator it) {
	this.session = session;
	oidDatabase = new SnmpOidDatabaseSupport();
		
	SnmpOid.setSnmpOidTable(oidDatabase);
	
	metaFinder = new MetaFinder(oidDatabase, it);

	v2Parameters = new SnmpParameters("public", "private");
	v2Parameters.setProtocolVersion(SnmpDefinitions.snmpVersionTwo);       
	agent.setParams(v2Parameters);
	profiles.put(V1_DEFAULT_PROFILE_NAME, V1_DEFAULT_PROFILE);
	profiles.put(V2_DEFAULT_PROFILE_NAME, V2_DEFAULT_PROFILE);
	profiles.put(V3_DEFAULT_PROFILE_NAME, V3_DEFAULT_PROFILE);
	this.host = host;
	this.port = port;
	try {
	    trapFrame = new TrapFrame("["+ host + ":" + port +
				      "] window traps.");
	}catch(Exception e) {
	    
	}
	int p = Integer.parseInt(port) + 1;
	try {
	    trapFrame.text.append("Listening traps [" + InetAddress.getLocalHost().getHostName() +":"+ p + "]\n");
	}catch(Exception e) {
	    e.printStackTrace();
	}
	
	trapListener = new TrapListenerImpl(trapFrame.text, this);
	informListener = new InformListenerImpl(trapFrame.text, this);
	
	trapAgent = (com.sun.management.snmp.manager.SnmpEventReportDispatcher) trapAgents.get(new Integer(p));
	try{
	    if(trapAgent == null) {
		trapAgent = 
		    new com.sun.management.snmp.manager.SnmpEventReportDispatcher(p);
		new Thread(trapAgent).start();
		trapAgents.put(new Integer(p), trapAgent);
	    }
	    trapAgent.addTrapListener(trapListener); 
	    trapAgent.addInformListener(informListener); 
	} catch(SocketException e) { 
	    e.printStackTrace();
	}
    }

    public SimpleManager(SnmpEngine eng,
			 SnmpPeer agent, 
			 SnmpSession session,
			 String host,
			 String port) throws SnmpStatusException
    {
	this.engine = eng;
	this.session = session;
	oidDatabase = new SnmpOidDatabaseSupport();
	SnmpOidTable[] tables = new SnmpOidTable[1];
	tables[0] = new RFC1213_MIBOidTable();
	
	SnmpOid.setSnmpOidTable(oidDatabase);

	metaFinder = new MetaFinder(oidDatabase, tables);
	metaFinder.start();
	v2Parameters = new SnmpParameters("public", "private");
	v2Parameters.setProtocolVersion(SnmpDefinitions.snmpVersionTwo);       
	agent.setParams(v2Parameters);
	profiles.put(V1_DEFAULT_PROFILE_NAME, V1_DEFAULT_PROFILE);
	profiles.put(V2_DEFAULT_PROFILE_NAME, V2_DEFAULT_PROFILE);
	profiles.put(V3_DEFAULT_PROFILE_NAME, V3_DEFAULT_PROFILE);
	try {
	    trapFrame = new TrapFrame("["+InetAddress.getLocalHost()+"] window traps.");
	    helpFrame = new HelpFrame("easymanager help window.");
	    
	}catch(UnknownHostException e) {
	    
	}
	trapListener = new TrapListenerImpl(trapFrame.text, this);
	informListener = new InformListenerImpl(trapFrame.text, this);
	try{
	    DaemonTaskServer d = new DaemonTaskServer();
	    trapAgent = 
		new com.sun.management.snmp.manager.SnmpEventReportDispatcher(Integer.parseInt(port)+1);
	} catch(SocketException e) { e.printStackTrace(); }
	new Thread(trapAgent).start();
	
	this.host = host;
	this.port = port;
	try{
	    boolean completed = false;
	    InputStreamReader i = new InputStreamReader(System.in);
	    BufferedReader in = new BufferedReader(i);
	    while(true) {
		System.out.println("Start making your request:(h for help window)");
		String req = in.readLine();
		try{
		    if(req.equals("trap")) {
			trapAgent.addTrapListener(trapListener); 
			trapAgent.addInformListener(informListener); 
			trapFrame.show();
			continue;
		    }
		    if(req.equals("untrap")) { 
			trapAgent.removeTrapListener(trapListener); 
			trapAgent.removeInformListener(informListener); 
			trapFrame.hide();
			continue;
		    }
		    if(req.startsWith("loadmib")) {
			handleLoadMib(req);
			// TO DO handleClassBrowser();
			continue;
		    }
		    if(req.startsWith("loadnhas2")) {
			handleLoadMib("loadmib com.sun.nhas.ma.snmp.NHAS_STATISTICS_MIBOidTable");
		    continue;
		    }
		    if(req.equals("printmib")) {
			handlePrintMib();
			continue;
		    }
		    if(req.startsWith("discovermib")) {
			handleDiscoverMib(req);
			continue;
		    }

		    if(req.equals("v3connect")) {
			handleConnectV3();
			continue;
		    }
		    if(req.startsWith("addprofile")) {
			handleAddProfile(req);
			continue;
		    }
		    if(req.startsWith("remprofile")) {
			handleRemProfile(req);
			continue;
		    }
		    if(req.startsWith("printprofile")) {
			handlePrintProfile(req);
			continue;
		    }
		    if(req.equals("printallprofile")) {
			handlePrintAllProfile();
			continue;
		    }
		    if(req.equals("exit")) System.exit(0);
		    if(req.equals("h") || req.equals("help")) { 
			printHelp();
			continue;
		    }
		    parse(req);
		}
		catch(Exception parseException) 
		    {
			//parseException.printStackTrace();
			System.out.println("Badly formed request!!!");
			continue;
		    }
		//Handle security parameters
		boolean security = false;
		String[] params = null;
		int start = req.indexOf(':');
		int end = req.lastIndexOf(':');
		int cpt = 0;
		String sub = null;
       
		if((end - start) > 2) {
		    sub = req.substring(start + 1, end);
		   
		}
		
		
		if(protocol.startsWith("1")){
		    
		    agent.getParams().
			setProtocolVersion(SnmpDefinitions.snmpVersionOne);
		    session.snmpOptions.setPduFixedOnError(false);
		    session.setDefaultPeer(agent);
		    if(sub != null) {
			StringTokenizer token = new StringTokenizer(sub);
			//The protocol version is removed
			token.nextToken();
			if(token.countTokens() != 0) {
			    token = handleProfile(token, sub);
			    while(token.hasMoreTokens()) {
				String flag = token.nextToken();
				if(flag.equals("-r")) {
				    ((SnmpParameters)agent.getParams()).setRdCommunity(token.nextToken());
				    continue;
				}
				if(flag.equals("-w")) {
				    ((SnmpParameters)agent.getParams()).setWrCommunity(token.nextToken());
				    continue;
				}
				
			    }
			}
		    }
		}
		else
		    if(protocol.startsWith("2")) {
			
			agent.getParams().
			    setProtocolVersion(SnmpDefinitions.snmpVersionTwo);
			session.setDefaultPeer(agent);
			if(sub != null) {
			    StringTokenizer token = new StringTokenizer(sub);
			    //The protocol version is removed
			    token.nextToken();
			    if(token.countTokens() != 0) {
				token = handleProfile(token, sub);
				while(token.hasMoreTokens()) {
				    String flag = token.nextToken();
				    if(flag.equals("-r")) {
					((SnmpParameters)agent.getParams()).setRdCommunity(token.nextToken());
					continue;
				    }
				    if(flag.equals("-w")) {
					((SnmpParameters)agent.getParams()).setWrCommunity(token.nextToken());
					continue;
				    }
				    
				}
			    }
			}
		    }else
			if(protocol.startsWith("3")) {
			    //Lazy connection
			    try {
				handleConnectV3();
			    }catch(SnmpStatusException e) {
				continue;
			    }
			    
			    session.setDefaultPeer(v3agent);
			    SnmpUsmParameters p = null;
			    if(sub != null) {
				StringTokenizer token = new StringTokenizer(sub);
				//The protocol version is removed
				token.nextToken();
				
				if(token.countTokens() != 0) {
				    String contextName = null;
				    int flags = 0;
				    token = handleProfile(token, sub);
				    while(token.hasMoreTokens()) {
					String flag = token.nextToken();
					if(flag.equals("-u")) {
					    p = new SnmpUsmParameters(engine, 
								      token.nextToken());
					    continue;
					}
					if(flag.equals("-f")) {	
					    String sec = token.nextToken();
					    if(sec.equals("noAuthNoPriv")) {
						flags = SnmpDefinitions.noAuthNoPriv;
						continue;
					    }
					    if(sec.equals("authNoPriv")) {
						flags = SnmpDefinitions.authNoPriv;
						continue;
					    }
					    if(sec.equals("authPriv")) {
						flags = SnmpDefinitions.authPriv;
						continue;
					    }
					    
					}
					if(flag.equals("-r")) {
					    flags |= SnmpDefinitions.reportableFlag;
					    continue;
					}
					
					if(flag.equals("-c")) {	
					    contextName = token.nextToken();
					    continue;
					}
				    }
				    if(p == null) 
					p = new SnmpUsmParameters(engine);
				    
				    p.setSecurityLevel(flags);
				    if(contextName != null)
					p.setContextName(contextName.getBytes());
				    
				}
			    } else
				p = new SnmpUsmParameters(engine);
			    p.setContextEngineId(v3agent.getEngineId().getBytes());
			    v3agent.setParams(p); 

			}
		
		SnmpRequest request = null; 
		SnmpVarBindList l = new SnmpVarBindList(list);
		switch(f) {
		case 0:
		    request = session.snmpGetRequest(null, l);
		    System.out.println("Send get request to SNMP agent on " + 
				       host + " at port " + port);
		    break;
		case 1:
		    request = session.snmpGetNextRequest(null, l);
		    System.out.println("Send getNext req to SNMP agent on " + 
				       host + " at port " + port);
		    break;
		case 2:
		    request = session.snmpSetRequest(null, l);
		    System.out.println("Send set request to SNMP agent on " + 
				       host + " at port " + port);
		    break;
		case 3:
		    request = session.snmpGetBulkRequest(null, 
							 l, 
							 Integer.
							 parseInt(nrp),
							 Integer.
							 parseInt(max));
		    System.out.println("Send getBulk req to SNMP agent on " + 
				       host + " at port " + port);
		    break;
		}

		completed = request.waitForCompletion(10000);
		
		if (completed == false) {
		    System.out.println("Request timed out.");
		    
		    System.out.println("Request: " + request.toString());
		}
		
		// Now we have a response. 
		//Check if the response contains an error.
		//
		int errorStatus = request.getErrorStatus();
		if (errorStatus != SnmpDefinitions.snmpRspNoError) {
		    System.out.println("**********************" + 
				       "******************");
		    System.out.println("ERROR status = " + 
				       SnmpRequest.
				       snmpErrorToString(errorStatus));
		    System.out.println("ERROR index = " + 
				       request.getErrorIndex());
		    System.out.println("***********************" + 
				       "*****************");
		    
		}
		else {
		    // Now we shall display the content of the result.
		    //
		    if(request.isReport()) {
			SnmpVarBindList result = request.getResponseVarBindList();
			System.out.println("**********************" + 
					   "******************");
			System.out.println("WARNING REPORT: \n" + result);
			System.out.println("***********************" + 
					   "*****************");
		    }
		    else {
			SnmpVarBindList result = request.getResponseVarBindList();
			System.out.println("**********************" + 
					   "******************");
			System.out.println("RESULT: \n" + result);
			System.out.println("***********************" + 
					   "*****************");
		    }
		}
		agent.setParams(v2Parameters);
	    }
	    
	} catch(Exception e) {
	    System.err.println("Exception occurred:" + e );
	    //e.printStackTrace();
	}
    }
    
    private void handleAddProfile(String req) {
	StringTokenizer t = new StringTokenizer(req);
	t.nextToken();
	String key = t.nextToken();
	String profile = req.substring(req.indexOf(key) + key.length() + 1);
	profiles.put(key, profile);
    }
    
    private void handleRemProfile(String req) {
	StringTokenizer t = new StringTokenizer(req);
	t.nextToken();
	String key = t.nextToken();
	try {
	    profiles.remove(key);
	}catch(Exception e) {
	    throw new IllegalArgumentException("Unknown profile ["+key+"]");
	}
    }
    private void handlePrintAllProfile() {
	for(Enumeration e = profiles.keys(); e.hasMoreElements();) {
	    String key = (String) e.nextElement();
	    String params = (String) profiles.get(key);
	    java.lang.System.out.println(key + " : " + params);
	}
    }

    private void handlePrintProfile(String req) {
	StringTokenizer t = new StringTokenizer(req);
	t.nextToken();
	String key = t.nextToken();
	String params = (String) profiles.get(key);
	if(params == null) return;
	java.lang.System.out.println(key + ":" + params);
    }
    
    private SnmpPeer asyncProfile2Peer(String prof) 
	throws SnmpStatusException, UnknownHostException {
	String params = (String) profiles.get(prof);
	if(params == null) 
	    throw new IllegalArgumentException("Unknown profile");
	StringTokenizer token = new StringTokenizer(params);
	String protocol = token.nextToken();
	if(protocol.equals("1"))
	    return v1Peer(token);
	if(protocol.equals("2"))
	    return v2Peer(token);
	if(protocol.equals("3"))
	    return v3Peer(token);
	return null;
    }
    
    SnmpPeer v1v2Peer(StringTokenizer token,
		      int vers) throws SnmpStatusException,
		      UnknownHostException {
	SnmpPeer p = new SnmpPeer(host, Integer.parseInt(port));
	SnmpParameters params = new SnmpParameters();
	params.setProtocolVersion(vers);
	
	while(token.hasMoreTokens()) {
	    String arg = token.nextToken();
	    if(arg.equals("-r")) {
		params.setRdCommunity(token.nextToken());
		continue;
	    }
	    if(arg.equals("-w")) {
		params.setWrCommunity(token.nextToken());
		continue;
	    }
	}
	
	p.setParams(params);
	return p;
    }
    
    SnmpPeer v2Peer(StringTokenizer token) throws SnmpStatusException, UnknownHostException {
	return v1v2Peer(token, SnmpDefinitions.snmpVersionTwo);
    }
    
    SnmpPeer v1Peer(StringTokenizer token) throws SnmpStatusException, 
    UnknownHostException {
	return v1v2Peer(token, SnmpDefinitions.snmpVersionOne);
    }
    
    SnmpPeer v3Peer(StringTokenizer token) throws SnmpStatusException, 
    UnknownHostException {
	SnmpUsmPeer p = null;
	SnmpUsmParameters params = null;
	try {
	    p = new SnmpUsmPeer(engine, host, Integer.parseInt(port));

	    handleConnectV3(p);
	    params = new SnmpUsmParameters(engine);
	}catch(Exception e) {
	    throw new SnmpStatusException(e.getMessage());
	}
	while(token.hasMoreTokens()) {
	    String arg = token.nextToken();
	    if(arg.equals("-u")) {
		params.setPrincipal(token.nextToken());
		continue;
	    }
	    if(arg.equals("-f")) {
		String sec = token.nextToken();
		if(sec.equals("authPriv")) {
		    params.setSecurityLevel(SnmpDefinitions.authPriv);
		    continue;
		}
		if(sec.equals("authNoPriv")) {
		    params.setSecurityLevel(SnmpDefinitions.authNoPriv);
		    continue;
		}
		if(sec.equals("noAuthNoPriv")) {
		    params.setSecurityLevel(SnmpDefinitions.noAuthNoPriv);
		    continue;
		}
	    }
	    if(arg.equals("-c")) {
		params.setContextName(token.nextToken().getBytes());
		continue;
	    }
	}
	p.setParams(params);
	return p;
    }
    
    public void handleDiscoverMib() throws SnmpStatusException, UnknownHostException {
	handleDiscoverMib(" aaa ");
    }

    private void handleDiscoverMib(String req) throws SnmpStatusException, UnknownHostException {
	StringTokenizer t = new StringTokenizer(req);
	t.nextToken();
	String start = null;
	String stop = null;
	String prof = null;
	while(t.hasMoreTokens()) {
	    String param = t.nextToken();
	    if(param.equals("-p")) {
		prof = t.nextToken();
		continue;
	    }
	    if(param.equals("-start")) {
		start = t.nextToken();
		continue;
	    }
	    if(param.equals("-stop")) {
		stop = t.nextToken();
		continue;
	    }
	}
	if(prof == null)
	    prof = V2_DEFAULT_PROFILE_NAME;
	SnmpPeer p = asyncProfile2Peer(prof);
	if(start == null)
	    start = "1.1";
	if(stop == null)
	    stop = "2.1";
	String key = start + "-" + stop;
	MibFrame frame = (MibFrame) mibFrames.get(key);
	if(frame == null) {
	    frame = new MibFrame(this,
				 p,
				 session,
				 start,
				 stop,
				 trapFrame);
	    mibFrames.put(key, frame);
	} else
	    frame.setPeer(p);
	
	frame.setProfile(prof);
	
	frame.display();
	frame.resync();
	
    }
    
    private StringTokenizer handleProfile(StringTokenizer token, 
					  String sub) {
	//Handle profile
	String prof = token.nextToken();
	String profKey = null;
	String params = null;
	if(prof.equals("-p")) {
	    profKey = token.nextToken();
	    if(profKey == null)
		throw new IllegalArgumentException("Unknown profile");
	    params = (String) profiles.get(profKey);
	    if(params == null) 
		throw new IllegalArgumentException("Unknown profile");
	    token = new StringTokenizer(params);
	}
	else {
	    token = new StringTokenizer(sub);
	    //The protocol version is removed
	    token.nextToken();
	}
	return token;
    }
    
    private void handleLoadMib(String req) {
	StringTokenizer t = new StringTokenizer(req);
	t.nextToken();
	String mib = t.nextToken();
	try {
	    SnmpOidTableSupport obj = 
		(SnmpOidTableSupport) (Class.forName(mib)).newInstance();
	    oidDatabase.add(obj);
	    
	}catch(Exception e) {
	    System.err.println(e);
	}   
    }
    
    private void handlePrintMib() {
	Enumeration e = oidDatabase.getAllEntries().elements();
	for(; e.hasMoreElements();) {
	    SnmpOidRecord rec = (SnmpOidRecord) e.nextElement();
	    System.out.println("NAME :" + rec.getName() + " OID :" + rec.getOid()+" TYPE : " + rec.getType());
	}
    }
    
    private void handleConnectV3(SnmpUsmPeer peer) throws SnmpStatusException {
	SnmpUsmParameters p = null;
	try {
	    try {
		p = new SnmpUsmParameters(engine, "defaultUser");
	    }catch(Exception e) {
		return;
	    }
	    
	    p.setContextEngineId(peer.getEngineId().getBytes());
	    p.setSecurityLevel(SnmpDefinitions.authNoPriv);
	    peer.setParams(p);
	    peer.processUsmTimelinessDiscovery();
	    p.setSecurityLevel(SnmpDefinitions.authNoPriv);
	    System.out.println("*** Connection succeeded.");
	}catch(SnmpStatusException e) {
	    System.out.println("***EXCEPTION. Unable to activate Usm discovery. Check that the agent is running.");
	    throw e;
	}
    }

    private void handleConnectV3() throws SnmpStatusException {
	if(!connected) {
	    try {
		v3agent = new SnmpUsmPeer(engine, host, Integer.parseInt(port));
	    }catch(UnknownHostException e) {
		System.out.println("*** Connection Failed.");
		System.out.println("Wrong host name : [" + host + "]");	
	    }
	    catch(Exception e) {
		throw new SnmpStatusException(e.getMessage());
	    }
	    handleConnectV3(v3agent);
	    connected = true;
	}
    }
    
    public static void main(String argv[]) {
	
	String host = "localhost";
	String port = "8085";
	boolean crypto = false;
	//Parse args
	for(int i = 0; i < argv.length; i++) {
	    if(argv[i].equals("-host")) {
		host = argv[i+1];
		continue;
	    }
	    if(argv[i].equals("-port")) {
		port = argv[i+1];
		continue;
	    }
	    if(argv[i].equals("-crypto")) {
		crypto = true;
		continue;
	    }
	}
	// Initialize trace property.
        // 
        try {
            TraceManager.parseTraceProperties();
            Trace.send(Trace.LEVEL_TRACE, 
		       Trace.INFO_MISC, 
		       "Simple Manager", 
		       "main", 
		       "Default trace is on");
        } catch(java.io.IOException e) {
            e.printStackTrace();
        }
	SnmpEngineParameters p = null;
	//Test if cryptographie is activated
	if(crypto) {
	    p = new SnmpEngineParameters();
	    p.activateEncryption();
	    System.out.println("Encryption activated.");
	}
	
	SnmpSession session = null;
	try {
	    session = new SnmpSession(p, 
				      null, 
				      "Simple Manager session", 
				      null);
	}catch(Exception e) {
	    e.printStackTrace();
	    return;
	}
	
	try {
	    SnmpPeer agent = new SnmpPeer(host, Integer.parseInt(port));
	    session.setDefaultPeer(agent);
	    java.lang.System.out.println("easymanager initialized. SNMP Agent is [" + host + ":" + port +"]");
	    new SimpleManager(session.getEngine(), agent, session, host, port);
	}catch(Exception e)
	    { e.printStackTrace(); }
    }
    
    /**
     * Return usage of the program.
     */
    public static void  usage() {
        java.lang.System.out.println("java SimpleManager <host> <port> {crypto}");
        java.lang.System.out.println("where");
        java.lang.System.out.println("\t-host: hostname or IP address of the SNMP agent you want to query.");
        java.lang.System.out.println("\t-port: port number to use.");
	java.lang.System.out.println("\t-crypto: activate encryption.");
    }
    
    private void printHelp() {
	helpFrame.display();
	
    }
}
