/*
 * @(#)file      XMLMibValidationHandler.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.9
 * @(#)lastedit  04/05/10
 * 
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package validator;

// JAXP packages
import org.w3c.dom.*;

import java.util.ArrayList;
import java.util.List;

import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;

import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpOidTable;

class XMLMibValidationHandler {

    private static final String VALIDATOR = "validator";
    private static final String VALIDATION = "validation";
    private static final String AGENT = "agent";
    private static final String META = "meta";
    private static final String SPECIFIC_REQUESTS = "specificrequests";
    private static final String SPECIFIC_REQUESTS_GETBULK = "getbulk";
    private static final String GETBULK_START = "startoid";
    private static final String GETBULK_NBREPEATS = "nbrepeaters";
    private static final String SPECIFIC_REQUESTS_GET = "get_after_getnext";
    private static final String TREELIMITS = "treelimits";
    private static final String OIDTABLE = "oidtable";
    private static final String OIDTABLE_EXCLUDELIST = "excludelist";
    private static final String EXCLUDELIST_EXCLUDED = "exclude";
    private static final String TABLES = "tables";
    private static final String TABLE_INFO = "tableinfo";
    private static final String TABLE_INFO_DESCRIPTION = "description";
    private static final String TABLE_INFO_OID = "oid";
    private static final String TABLE_INFO_ALLOW_EMPTY = "allowempty";
    private static final String TABLE_INFO_NB_ENTRIES = "nbentries";
    private static final String TABLE_INFO_INDEXES = "indexlist";
    private static final String TABLE_INDEX = "index";
    private static final String TABLE_INFO_STRICT = "strict";
    private static final String TABLE_INFO_NB_COLUMNS = "nbcolumns";
    private static final String TREELIMITS_START = "startoid";
    private static final String TREELIMITS_STOP = "stopoid";
    private static final String TREELIMITS_REQUEST = "request";
    private static final String AGENT_ADDRESS = "host";
    private static final String AGENT_PORT = "port";
    private static final String AGENT_VERSION = "version";
    private static final String VALIDATION_NAME = "name";
    private static final String VALIDATION_DESCRIPTION = "description";
    private static final String VALIDATION_TYPE = "type";
    private static final String VALIDATION_THREAD = "thread";
    private static final String VALIDATION_ITERATION = "iteration";
    private static final String OIDTABLE_CLASS = "class";
    private static final String SNMP_VERSION_1 = "v1";
    private static final String SNMP_VERSION_2 = "v2";
    ValidatorLogger logger = null;

    XMLMibValidationHandler(ValidatorLogger logger) {
        this.logger = logger;
    }
    
    /**
     * Log common attributes of a DOM2 Node.
     */
    private void logCommon(Node n) {
	StringBuffer buff = new StringBuffer();
        buff.append(" nodeName=\"" + n.getNodeName() + "\"");
	
        String val = n.getNamespaceURI();
        if (val != null) {
            buff.append(" uri=\"" + val + "\"");
        }

        val = n.getPrefix();
        if (val != null) {
            buff.append(" pre=\"" + val + "\"");
        }

        val = n.getLocalName();
        if (val != null) {
            buff.append(" local=\"" + val + "\"");
        }

        val = n.getNodeValue();
        if (val != null) {
            buff.append(" nodeValue=");
            if (val.trim().equals("")) {
                // Whitespace
                buff.append("[WS]");
            } else {
                buff.append("\"" + n.getNodeValue() + "\"");
            }
        }
	logger.debug("", buff.toString());
    }

    private void handleElement(Node n, Object ctxt) throws Exception {
	if(n.getNodeName().equals(VALIDATOR)) {
	}
	
	if(n.getNodeName().equals(VALIDATION)) {
	    
	    String name = null;
	    int nbthread = 0;
	    int iterat = 0;
	    String type = null;
	    String description = null;
	    NamedNodeMap atts = n.getAttributes();
	    for (int i = 0; i < atts.getLength(); i++) {
		Node att = atts.item(i);
		if(att.getNodeName().equals(VALIDATION_NAME)) 
		    name = att.getNodeValue();
		if(att.getNodeName().equals(VALIDATION_DESCRIPTION)) 
		    description = att.getNodeValue();
                if(att.getNodeName().equals(VALIDATION_THREAD))
		    nbthread = Integer.parseInt(att.getNodeValue());
		if(att.getNodeName().equals(VALIDATION_ITERATION))
		    iterat = Integer.parseInt(att.getNodeValue());
		if(att.getNodeName().equals(VALIDATION_TYPE))
		    type = att.getNodeValue();
	    }
	    SnmpMibValidation validation = 
		new SnmpMibValidation(name,
				      description,
				      type,
				      iterat,
				      nbthread);
	    
	    SnmpMibValidator validator = (SnmpMibValidator) ctxt;
	    validator.addValidation(validation);
	    ctxt = validation;
	}
	
	if(n.getNodeName().equals(SPECIFIC_REQUESTS)) {
	    
	}
	
	if(n.getNodeName().equals(SPECIFIC_REQUESTS_GETBULK)) {
	    String startoid = null;
	    int nbrepeaters = -1;
	    NamedNodeMap atts = n.getAttributes();
	    for (int i = 0; i < atts.getLength(); i++) {
		Node att = atts.item(i);
                if(att.getNodeName().equals(GETBULK_START))
		    startoid = att.getNodeValue();
		if(att.getNodeName().equals(GETBULK_NBREPEATS))
		    nbrepeaters = Integer.parseInt(att.getNodeValue());
	    }
	    SnmpMibValidation validation = (SnmpMibValidation) ctxt;
	    validation.setGetBulkRequest(new SnmpOid(startoid), nbrepeaters);
	}

	if(n.getNodeName().equals(SPECIFIC_REQUESTS_GET)) {
	    boolean b = Boolean.valueOf(n.getFirstChild().getNodeValue()).booleanValue();
	    SnmpMibValidation validation = (SnmpMibValidation) ctxt;
	    validation.setGetAfterGetNext(b);
	}

	if(n.getNodeName().equals(AGENT)) {
	    String ipAddress = null;
	    String version = null;
	    int port = 0;
	    NamedNodeMap atts = n.getAttributes();
	    for (int i = 0; i < atts.getLength(); i++) {
		Node att = atts.item(i);
		if(att.getNodeName().equals(AGENT_ADDRESS)) 
		    ipAddress = att.getNodeValue();
                if(att.getNodeName().equals(AGENT_PORT))
		    port = Integer.parseInt(att.getNodeValue());
		if(att.getNodeName().equals(AGENT_VERSION))
		    version = att.getNodeValue();
	    }
	    
	    SnmpPeer agent = new SnmpPeer(ipAddress, 
					  port);
	    final SnmpParameters params = 
		new SnmpParameters("public", 
				   "private");
	    
	    if(version.equals(SNMP_VERSION_1))
		params.setProtocolVersion(SnmpDefinitions.snmpVersionOne);
	    if(version.equals(SNMP_VERSION_2))
		params.setProtocolVersion(SnmpDefinitions.snmpVersionTwo);
	     
            agent.setParams(params);
	    
	    SnmpMibValidation validation = (SnmpMibValidation) ctxt;
	    validation.setPeer(agent);
	    //return ctxt;
	}
	
	if(n.getNodeName().equals(META)) {
	}
	
	if(n.getNodeName().equals(TREELIMITS)) {
	    String start = null;
	    String stop = null;
	    String request = null;
	    NamedNodeMap atts = n.getAttributes();
	    for (int i = 0; i < atts.getLength(); i++) {
		Node att = atts.item(i);
		if(att.getNodeName().equals(TREELIMITS_START)) 
		    start = att.getNodeValue();
		if(att.getNodeName().equals(TREELIMITS_STOP)) 
		    stop = att.getNodeValue();
		if(att.getNodeName().equals(TREELIMITS_REQUEST))
		    request = att.getNodeValue();
		
	    }
	    SnmpMibValidation validation = (SnmpMibValidation) ctxt;
	    validation.addTreeLimits(new SnmpOid(start), 
				     new SnmpOid(stop),
				     request);
	    //return ctxt;
	}
	
	if(n.getNodeName().equals(OIDTABLE)) {
	    String clss = null;
	    NamedNodeMap atts = n.getAttributes();
	    for (int i = 0; i < atts.getLength(); i++) {
		Node att = atts.item(i);
		if(att.getNodeName().equals(OIDTABLE_CLASS)) 
		    clss = att.getNodeValue();
	    }
	    SnmpOidTable t = (SnmpOidTable) Class.forName(clss).newInstance();
	    SnmpOid.setSnmpOidTable(t);
	    SnmpMibValidationOidTable table = new SnmpMibValidationOidTable(t);

	    SnmpMibValidation validation = (SnmpMibValidation) ctxt;
	    validation.addOidTable(table);
	    //return;
	    ctxt = table;
	}

	if(n.getNodeName().equals(OIDTABLE_EXCLUDELIST)) {
	    ArrayList excludeList = new ArrayList();

	    SnmpMibValidationOidTable table = 
		(SnmpMibValidationOidTable) ctxt;

	    table.setExcludeList(excludeList);
	    //return;
	    ctxt = excludeList;
	}
	
	if(n.getNodeName().equals(EXCLUDELIST_EXCLUDED)) {
	    ArrayList excludeList = (ArrayList) ctxt;
	    
	    excludeList.add(n.getFirstChild().getNodeValue());
	}

	if(n.getNodeName().equals(TABLES)) {
	    ArrayList tables = new ArrayList();
	    
	    SnmpMibValidation validation = (SnmpMibValidation) ctxt;
	    validation.setTableInfos(tables);
	    //return;
	    ctxt = tables;
	}
	
	if(n.getNodeName().equals(TABLE_INFO)) {
	    SnmpTableInfo description = new SnmpTableInfo();

	    ArrayList tables = (ArrayList) ctxt;
	    tables.add(description);
	    //return;
	    ctxt = description;
	}
	
	if(n.getNodeName().equals(TABLE_INFO_DESCRIPTION)) {
	    String description = n.getFirstChild().getNodeValue();
	    SnmpTableInfo info = (SnmpTableInfo) ctxt;
	    info.setDescription(description);
	    //return;
	}
	
	if(n.getNodeName().equals(TABLE_INFO_OID)) {
	    String oid = n.getFirstChild().getNodeValue();
	    SnmpTableInfo description = (SnmpTableInfo) ctxt;
	    description.setOid(new SnmpOid(oid));
	    //return;
	}
	
	if(n.getNodeName().equals(TABLE_INFO_ALLOW_EMPTY)) {
	    boolean allow = Boolean.valueOf(n.getFirstChild().getNodeValue()).booleanValue();
	    SnmpTableInfo description = (SnmpTableInfo) ctxt;
	    description.allowEmpty(allow);
	    //return;
	}

	if(n.getNodeName().equals(TABLE_INFO_NB_ENTRIES)) {
	    int nbentries = Integer.parseInt(n.getFirstChild().getNodeValue());
	    SnmpTableInfo description = (SnmpTableInfo) ctxt;
	    description.setNbEntries(nbentries);
	    //return;
	}

	if(n.getNodeName().equals(TABLE_INFO_INDEXES)) {
	    List expectedIndexes = new ArrayList();
	    SnmpTableInfo description = (SnmpTableInfo) ctxt;
	    description.setExpectedIndexes(expectedIndexes);
	    //return;
	    ctxt = expectedIndexes;
	}
	
	if(n.getNodeName().equals(TABLE_INDEX)) {
	    String index = n.getFirstChild().getNodeValue();
	    List l = (List) ctxt;
	    l.add(index);
	}
	
	if(n.getNodeName().equals(TABLE_INFO_STRICT)) {
	    boolean strict = Boolean.valueOf(n.getFirstChild().getNodeValue()).booleanValue();
	    SnmpTableInfo description = (SnmpTableInfo) ctxt;
	    description.strict(strict);
	}

	if(n.getNodeName().equals(TABLE_INFO_NB_COLUMNS)) {
	    int nbcolumns = Integer.parseInt(n.getFirstChild().getNodeValue());
	    SnmpTableInfo description = (SnmpTableInfo) ctxt;
	    description.setNbColumns(nbcolumns);
	}
	
	for (Node child = n.getFirstChild(); child != null;
	     child = child.getNextSibling()) {
	    handleElement(child, ctxt);
	}
    }
    
    public void handle(Node n, Object ctxt) throws Exception {
	handleElement(n, ctxt);
    }
}
