/*
 * @(#)file      XMLSchemaDOMParser.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)lastedit  04/05/10
 * 
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package validator;

// JAXP packages
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;

import java.io.File;


abstract class XMLSchemaDOMParser {
    /** All output will use this encoding */
    static final String outputEncoding = "UTF-8";

    /** Constants used for JAXP 1.2 */
    static final String JAXP_SCHEMA_LANGUAGE =
        "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA =
        "http://www.w3.org/2001/XMLSchema";
    static final String JAXP_SCHEMA_SOURCE =
        "http://java.sun.com/xml/jaxp/properties/schemaSource";

    private File schema = null;
    private ErrorHandler errorHandler = null;

    protected XMLSchemaDOMParser(File schema,
				 ErrorHandler errorHandler) {
	this.schema = schema;
	this.errorHandler = errorHandler;
    }
    
    protected Document parse(File docu) throws Exception {
        boolean ignoreWhitespace = false;
        boolean ignoreComments = false;
        boolean putCDATAIntoText = false;
        boolean createEntityRefs = false;
	
        // Step 1: create a DocumentBuilderFactory and configure it
        DocumentBuilderFactory dbf =
            DocumentBuilderFactory.newInstance();
	
        // Set namespaceAware to true to get a DOM Level 2 tree with nodes
        // containing namesapce information.  This is necessary because the
        // default value from JAXP 1.0 was defined to be false.
        dbf.setNamespaceAware(true);
	
        // Set the validation mode to either: no validation, DTD
        // validation, or XSD validation
        dbf.setValidating(true);

	try {
	    dbf.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
	} catch (IllegalArgumentException x) {
	    throw new Exception("Error: JAXP DocumentBuilderFactory attribute"+
				" not recognized:" + JAXP_SCHEMA_LANGUAGE);
	}
	
	dbf.setAttribute(JAXP_SCHEMA_SOURCE, schema);

        // Optional: set various configuration options
        dbf.setIgnoringComments(ignoreComments);
        dbf.setIgnoringElementContentWhitespace(ignoreWhitespace);
        dbf.setCoalescing(putCDATAIntoText);
        // The opposite of creating entity ref nodes is expanding them inline
        dbf.setExpandEntityReferences(createEntityRefs);

        // Step 2: create a DocumentBuilder that satisfies the constraints
        // specified by the DocumentBuilderFactory
        DocumentBuilder db = dbf.newDocumentBuilder();
	
        db.setErrorHandler(errorHandler);
	
	// Step 3: parse the input file
	Document doc = db.parse(docu);
	
        return doc;
    }
}
