/*
 * @(#)file      SnmpIpDomainsAddress.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */

package com.sun.management.snmp.rfc2573.internal;

import java.net.InetAddress;
import java.util.StringTokenizer;
import com.sun.management.snmp.rfc2573.target.SnmpAddress;

public class SnmpIpDomainsAddress implements SnmpAddress {
    public static final String UDP_DOMAIN = "snmpUDPDomain";
    public static final String TCP_DOMAIN = "snmpTCPDomain";
    public static final String UDP_DOMAIN_OID = "1.3.6.1.6.1.1";
    public static final String TCP_DOMAIN_OID = null;
    protected byte[] address = null;
    protected String strAddress = null;
    protected int port = -1;
    protected byte[] snmpAddress = null;
    protected String domain = null;
    protected String domainOid = null;
    
    public String getDomain() {
	return domain;
    }

    public String getDomainOid() {
	return domainOid;
    }

    public String getStringAddress() {
	return strAddress;
    }

    public String toString() {
	return strAddress + "/" + port;
    }
    
    public byte[] getMibAddress() {
	return snmpAddress;
    }
    
    public int getPort() {
	return port;
    }

    public byte[] getAddress() {
	return address;
    }

    /**
     * Handle RFC 1906 byte format.
     */
    public SnmpIpDomainsAddress(String domain, Byte[] address) 
	throws Exception {
	handleDomain(domain);
	translateAddress(address);
	translateAddress(this.address, port);
    }
    
    /**
     * Handle "ipaddress:port" string format.
     */
    public SnmpIpDomainsAddress(String domain, String address) 
	throws Exception {
	StringTokenizer token = new StringTokenizer(address, "/");
	this.strAddress = token.nextToken();
	this.address = InetAddress.getByName(strAddress).getAddress();
	this.port = Integer.parseInt(token.nextToken());
	
	handleDomain(domain);
	translateAddress(this.address, port);
    }
    
    /**
     * Overload if you want to handle new IP address format. This method must update address and port field. Domain and DomainOid field must be updated.
     */
    protected void handleDomain(String domain) {
	if(domain.equals(UDP_DOMAIN) || 
	   domain.equals(UDP_DOMAIN_OID)) {
	    this.domain = UDP_DOMAIN;
	    this.domainOid = UDP_DOMAIN_OID;
	}
	
	if(domain.equals(TCP_DOMAIN) ||
	   domain.equals(TCP_DOMAIN_OID)) {
	    this.domain = TCP_DOMAIN;
	    this.domainOid = TCP_DOMAIN_OID;
	}
    }
    
    /**
     * Overload if you want to handle new IP address format. This method must update address and port field.
     */
    protected void translateAddress(byte[] address, 
				    int port) throws Exception {
	snmpAddress = new byte[address.length + 4];
	snmpAddress[0] = (byte) (port & 0xFF);
	snmpAddress[1] = (byte) ((port & 0xFF00) >> 8);
	snmpAddress[2] = (byte) ((port & 0xFF0000) >> 16);
	snmpAddress[3] = (byte) ((port & 0xFF000000) >> 24);
	for(int i = 4; i < address.length + 4; i++) {
	    snmpAddress[i] = address[i - 4];
	}
    }
    
    /**
     * Overload if you want to handle new IP address format. This method must update address and port field.
     */
    protected void translateAddress(Byte[] address) throws Exception {
	int addrSize = address.length - 4;
	
	port = address[0].byteValue() & 0xFF;
	port |= ((int) (address[1].byteValue() & 0xFF)) << 8;
	port |= ((int) (address[2].byteValue() & 0xFF)) << 16;
	port |= ((int) (address[3].byteValue() & 0xFF)) << 24;
	this.address = new byte[addrSize];
	for(int i = 0; i < addrSize; i++)
	    this.address[i] = address[i + 4].byteValue();
	
	InetAddress a = InetAddress.getByAddress(this.address);
	strAddress = a.getHostAddress();
    }
}
