/*
 * @(#)file      SnmpPersistRowFileReader.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.FileWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import com.sun.jdmk.trace.Trace;

import com.sun.management.internal.snmp.SnmpPersistRowFileConsumer;
import com.sun.management.internal.snmp.SnmpPersistRowFile;


public class SnmpPersistRowFileReader {
    SnmpPersistRowFile file = null;
    String strFile = null;
    boolean init = false;
    Hashtable objs = new Hashtable();
    SnmpPersistRowFileConsumer consumer = null;
    String rowKey = null;
    public SnmpPersistRowFileReader(String file, 
				    String rowKey) {
	this.rowKey = rowKey;
	this.strFile = file;
    }
    
    public void init(SnmpPersistRowFileConsumer consumer) {
	this.consumer = consumer;
	init(strFile);	
    }
    
    public Object getObject(String key) {
	return objs.get(key);
    }

    public Enumeration getObjects() {
	return objs.elements();
    }
    
    public void putObject(Object key, Object obj) {
	objs.put(key, obj);
    }
    
    public void removeObject(Object key) throws IllegalArgumentException {
	Object obj = objs.remove(key);
	if(obj == null)
	    throw new IllegalArgumentException("Key is Unknown :" + key);
    }
    
    private void init(String f) {
	try {
	    this.file = new SnmpPersistRowFile(f,
					       rowKey,
					       ",",
					       consumer);
	    file.read();
	} catch(FileNotFoundException e) {
            if(isDebugOn()) 
                debug("Constructor", "No file. Will not read or persist data");
        }
        catch(IOException e) {
            if(isDebugOn()) 
                debug("Constructor", "IO error :" + e);
        }
	catch(Exception e) {
	    if(isDebugOn()) 
                debug("Constructor", "IO error :" + e);
	}
    }
    
    public void flushInFile() {
	if(file == null) {
	    if(isDebugOn())
		debug("flushInFile","No file, not persisting");
	    return;
	}
	try {
	    FileWriter writer = file.createWriter();
	    for (Enumeration e = objs.elements() ; e.hasMoreElements() ;) {
		Object obj = e.nextElement();
		file.write(writer, obj.toString() + "\n");
	    }
	    file.releaseWriter(writer);
	}catch(Exception e) {
	    if(isDebugOn()) {
		debug("flushInFile",e.toString());
	    }
	}
    }
    
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpPersistRowFileReader"; 
}
