/*
 * @(#)file      SnmpParamsReader.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.target;

import com.sun.jdmk.trace.Trace;

import com.sun.management.internal.snmp.SnmpPersistRowFileConsumer;
import com.sun.management.snmp.rfc2573.internal.SnmpPersistRowFileReader;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;

import java.util.Enumeration;

class ParamsConsumer implements SnmpPersistRowFileConsumer {
    //Define contexts
    static final int PARAMS_NAME = 0;
    static final int MPM = 1;
    static final int SECURITY_MODEL = 2;
    static final int SECURITY_NAME = 3;
    static final int SECURITY_LEVEL = 4;
    static final int STORAGE_TYPE = 5;

    SnmpParamsData params = new SnmpParamsData();
    SnmpParamsReader reader = null;
    ParamsConsumer(SnmpParamsReader reader) {
	this.reader = reader;
    }
    void reset() {
	params.paramsName = null;
	params.mpm = 0;
	params.securityModel = -1;
	params.securityName = null;
	params.securityLevel = -1;
	params.storageType = 0;
    }

    public Object rowBegin(String row, int line) {
	reset(); 
	return params;
    }
    
    public void rowEnd(Object rowContext, String row, int line) 
	throws Exception  {
	SnmpParamsData data = (SnmpParamsData) rowContext;
	reader.rowRead(data, row, line);
    }
    
    public void treatToken(String row,
			   int lineNumber,
			   String token,
			   int context,
			   Object rowContext,
			   boolean err) throws Exception {
	SnmpParamsData params = (SnmpParamsData) rowContext;
	if( err || (params == null) ) 
	    throw new Exception("Badly formed param entry. Line ["+
				lineNumber+"], data [" + row+"]");
	try {
	    switch(context) {
	    case PARAMS_NAME:
		params.paramsName = token;
		break;
	    case MPM:
		params.mpm = Integer.parseInt(token);
		break;
	    case SECURITY_MODEL:
		params.securityModel = Integer.parseInt(token);
		break;
	    case SECURITY_NAME:
		params.securityName = token;
		break;
	    case SECURITY_LEVEL:
		params.securityLevel = Integer.parseInt(token);
		break;
	    case STORAGE_TYPE:
		params.storageType = Integer.parseInt(token);
		break;
	    default:
	    }
	}catch(NumberFormatException e) {
	    throw new Exception(e.getMessage()+
				" Line :" + 
				lineNumber + 
				".\n" + 
				" Row : [" + 
				row +"]");
	}
    }
}

class SnmpParamsReader extends SnmpPersistRowFileReader {
    
    public SnmpParamsReader(String file) {
	super(file, "paramsEntry");
	init(new ParamsConsumer(this));
    }
    
    public SnmpParamsData getParams(String name) {
	return (SnmpParamsData) getObject(name);
    }
    
    public Enumeration getParams() {
	return getObjects();
    }
    
    void rowRead(SnmpParamsData d, String row, int line) throws Exception {
	addParams((SnmpParamsData) d.clone());
    }
    
    void addParams(SnmpParamsData data) {
	putObject(data.paramsName, data);
    }
    
    void removeParams(String paramsName) 
	throws IllegalArgumentException {
	removeObject(paramsName);
	flushInFile();
    }
	
    void addParams(String paramsName,
		   int mpm,
		   int securityModel,
		   String securityName,
		   int securityLevel,
		   int storageType) {
	SnmpParamsData data = new SnmpParamsData();
	data.paramsName = paramsName;
	data.mpm = mpm;
	data.securityModel = securityModel;
	data.securityName = securityName;
	data.securityLevel = securityLevel;
	data.storageType = storageType;
	addParams(data);
	flushInFile();
    }

    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpParamsReader"; 
}
