/*
 * @(#)file      SnmpPeerFactory.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.12
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.target;

import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.usm.SnmpUsm;
import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.rfc2573.internal.SnmpIpDomainsAddress;
import com.sun.management.snmp.rfc2573.target.SnmpAddress;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;

public class SnmpPeerFactory {
    public SnmpPeerFactory() {
    }
    /**
     * Overload this method in order to instanciate your own authoritative peer. This method creates <CODE> SnmpPeer </CODE> for SNMP V1 and SNMP V2. <CODE> SnmpUsmPeer</CODE> for SNMP V3 and Usm.
     * @param mpm Message Processing Model.
     * @param securityModel Security model.
     * @param address Peer address.
     * @param port Peer port.
     * @return The peer. 
     */
    public SnmpPeer createAuthoritativePeer(SnmpEngine engine,
					    SnmpParamsData params,
					    SnmpAddress addr) 
	throws Exception {
	return createPeer(engine, params, addr, true);
    }
    /**
     * Overload this method in order to instanciate your own non authoritative peer. This method creates <CODE> SnmpPeer </CODE> for SNMP V1 and SNMP V2. <CODE> SnmpUsmPeer</CODE> for SNMP V3 and Usm.
     * @param mpm Message Processing Model.
     * @param securityModel Security model.
     * @param address Peer address.
     * @param port Peer port.
     * @return The peer. 
     */
    public SnmpPeer createNonAuthoritativePeer(SnmpEngine engine,
					       SnmpParamsData params,
					       SnmpAddress addr) 
	throws Exception {
	return createPeer(engine, params, addr, false);
    }
    /**
     * Overload this method in order to instanciate your own peer. This method creates <CODE> SnmpPeer </CODE> for SNMP V1 and SNMP V2. <CODE> SnmpUsmPeer</CODE> for SNMP V3 and Usm.
     * @param mpm Message Processing Model.
     * @param securityModel Security model.
     * @param address Peer address.
     * @param port Peer port.
     * @param authoritative True to create an authoritative peer, false to create a non authoritative.
     * @return The peer. 
     */
    protected SnmpPeer createPeer(SnmpEngine engine,
				  SnmpParamsData params,
				  SnmpAddress addr,
				  boolean authoritative) throws Exception {
	SnmpPeer peer = null;
	
	if(!(addr instanceof SnmpIpDomainsAddress)) return null;
	SnmpIpDomainsAddress ipaddr = (SnmpIpDomainsAddress) addr;
	String address = ipaddr.getStringAddress();
	int port = ipaddr.getPort();
	if(isDebugOn()) 
	    debug("createPeer","Creating SNMP version : mpm [" + params.mpm + "] security model [" + params.securityModel +"]");
	switch(params.mpm) {
	case SnmpDefinitions.snmpVersionOne:
	case SnmpDefinitions.snmpVersionTwo:
	    switch(params.securityModel) {
	    case SnmpDefinitions.snmpVersionOne:
	    case SnmpDefinitions.snmpVersionTwo:
		if(isDebugOn()) 
		    debug("createPeer","Creating SnmpPeer("+address+","+port+")");
		peer = new SnmpPeer(address, port);
		if(params.mpm == SnmpDefinitions.snmpVersionTwo) {
		    SnmpParameters p = new SnmpParameters();
		    p.setProtocolVersion(SnmpDefinitions.snmpVersionTwo);
		    peer.setParams(p);
		}
		
		break;
	    default:
	    }
	    break;
	case SnmpDefinitions.snmpVersionThree:
	    switch(params.securityModel) {
	    case SnmpUsm.ID:
		if(isDebugOn()) 
		    debug("createPeer","Creating SnmpUsmPeer("+address+","+port+")");
		if(authoritative)
		    peer =  new SnmpUsmPeer(engine, address, port, null);
		else
		    peer =  new SnmpUsmPeer(engine, address, port);
		SnmpUsmParameters usmp = new SnmpUsmParameters(engine);
		peer.setParams(usmp);

		usmp.setSecurityLevel(SnmpParamsData.translateMIBSecurityLevel(params.securityLevel));
		usmp.setPrincipal(params.securityName);
		break;
	    default:
	    }
	    break;
	default:
	}
	return peer;
    }
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpPeerFactory"; 
}
