/*
 * @(#)file      SnmpRfc2573TargetManager.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.14
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.manager.target;

// Util import
import java.util.Enumeration;
import java.net.InetAddress;
import java.net.UnknownHostException;

import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.manager.SnmpParams;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpV3Parameters;
import com.sun.management.snmp.SnmpOidRecord;
import com.sun.management.snmp.SnmpOidTable;
import com.sun.management.snmp.SnmpStatusException;


// jdmk import
import com.sun.jdmk.TraceManager;
import com.sun.jdmk.trace.Trace;

// Contrib MGR import
import com.sun.management.snmp.rfc2573.manager.internal.target.SnmpTargetsParamsHandler;
import com.sun.management.snmp.rfc2573.manager.internal.target.SnmpTargetData;
import com.sun.management.snmp.rfc2573.manager.internal.target.SnmpParamsData;

public class  SnmpRfc2573TargetManager {

    /**
     * Register Params
     * paramsEntry=<params name>, <mp model>, <security model>, 
     * <security name>, <security level (0|1|3)>, <storage>
     */

    public static void registerParams(SnmpSession session,
			       String paramsIndex,
			       int    processingModel,
			       int    securityModel,
			       String securityName,
			       int    securityLevel,
			       int    storage)
	throws SnmpStatusException {
       
	SnmpParamsData data = new SnmpParamsData();

	//Name index
	data.paramsName = paramsIndex;

	// Processing Model
	data.mpm = processingModel;

	// Security Model
	data.securityModel = securityModel;

	// Security Name
	data.securityName = securityName;

	// Security Level
	data.securityLevel = securityLevel;

	// Storage Type
	data.storageType=storage;

	SnmpTargetsParamsHandler.setParams(session, data);

    }

    /** 
     * Unregister Params
     */
    public static void unregisterParams(SnmpSession session, 
				 String paramsIndex) 
	throws SnmpStatusException {
	SnmpTargetsParamsHandler.removeParams(session, paramsIndex);
    }
			       
    /** 
     * Register Target
     * targetsEntry=<addr name>, <domain>, <address>, <time interval>, 
     * <retry>, <tag list>, <params>
     */

    public static void registerTarget(SnmpSession session,
			       String targetIndex,
			       String targetDomain,
			       String targetAddress,
			       int    targetTimeout,
			       int    targetRetry,
			       String targetTagList,
			       String paramsIndex,
			       int    storage) throws SnmpStatusException {

	SnmpTargetData target = new SnmpTargetData();

	//Name index
	target.targetName=targetIndex;

	// Target domain
	target.domain=targetDomain;

	// Target Address UDP format
	target.address=targetAddress;

	// Timeout
	target.timeout=targetTimeout;

	// Retry count
	target.retry=targetRetry;

	// Tag List
	target.tags=targetTagList;

	// Params to use
	target.params=paramsIndex;

	// Storage type
	target.storageType=storage;
	
	// Register
	SnmpTargetsParamsHandler.setTarget(session, target);
    }

   
    /** 
     * Unregister Target
     */

    public static void unregisterTarget(SnmpSession session,
				 String targetIndex) 
	throws SnmpStatusException {
	SnmpTargetsParamsHandler.removeTarget(session, targetIndex);
    }

    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
	if (isTraceOn()) {
	    Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
	}
    }

    void trace(String func, String info) {
	if (isTraceOn()) {
	    trace(dbgTag, func, info);
	}
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
	if (isDebugOn()) {
	    Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
	}
    }

    void debug(String clz, String func, Throwable exception) {
	if (isDebugOn()) {
	    Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
	}
    }

    void debug(String func, String info) {
	if (isDebugOn()) {
	    debug(dbgTag, func, info);
	}
    }
    
    void debug(String func, Throwable exception) {
        if (isDebugOn()) {
	    debug(dbgTag, func, exception);
	}
    }
    
    String dbgTag = "SnmpRfc2573TargetManager"; 
}
