/*
 * @(#)file      Agent.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.14
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
// java import
//
import java.net.InetAddress;
import java.net.UnknownHostException;
// RI import
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineParameters;
import com.sun.management.snmp.SnmpUnknownModelException;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpSession;

// jdmk import
//
import com.sun.jdmk.TraceManager;
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpAdaptorServer;
import com.sun.management.snmp.agent.SnmpTrap;

//Contribution imports
import com.sun.management.snmp.rfc2573.manager.SnmpV3AppliMibRegistration;

/**
 * The Agent class provides a simple example on how to use the SNMP protocol adaptor.
 * A subset of MIB II (RFC1213) is implemented. The MIB is loaded and initialized. 
 * As such you can now see the MIB using your favorite SNMP manager, 
 * or you can use a web browser and see the MIB through the HTML adaptor.
 *
 * In this example, the SNMP adaptor is started on port 8095, 
 * and the traps are sent to the port 8086, i.e a non standard ports for SNMP. 
 * As such you do not need to be root to start the agent.
 */

public class Agent {
private static class TrapGenerator  extends Thread {
	
	SnmpAdaptorServer server = null;
	SnmpVarBindList lgarbage = null;
	
	private void sendV2() {
	    
	    println("V2 TRAP to send\n");
	    
	    //Send a V2 trap.
	    try {
		server.snmpV2Trap(SnmpTrap.egpNeighborLossOid, lgarbage);
		println("Trap V2 sent!");  
	    } catch(Exception e) { 
		e.printStackTrace(); 
	    }
	}
	
	private void sendV1() {
	    
	    println("V1 TRAP to send\n");
	    
	    // Send a V1 trap.
	    try{
		server.snmpV1Trap(0,0,lgarbage);
		println("Trap V1 sent!");
	    } catch(Exception e) {
		e.printStackTrace(); 
	    }
	}
	
	public void run() {
	    int c = 0;
	    while(true) {
		if(c != '\n')
		    println("\nType 1 in order to launch snmp V1 traps," +
			    " 2 for snmp V2.");
		try {
		    c =  java.lang.System.in.read();
		    if(c == '\n') continue;
		} catch(Exception e) {}
		
		if(c == '1') { sendV1(); continue; }
		if(c == '2') { sendV2(); continue; }
		println("(1 or 2). Ctrl C if you want to stop.");
	    }
	}
	
	public TrapGenerator(SnmpAdaptorServer server) {
	    
	    this.server = server;
	    
	    //Initialize a list of Varbinds
	    lgarbage = new SnmpVarBindList();
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.0.0"), 
			    new SnmpString("Test values")));
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.1.0"), 
			    new SnmpNull()));
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.2.0"), 
			    new SnmpInt (43)));
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.3.0"), 
			    new SnmpString ("Test values")));
	    
	    //Instanciate an address OID varBind
	    SnmpVarBind vb = null;
	    try{
		final byte[] inet = 
		    InetAddress.getLocalHost().getAddress();
		final SnmpIpAddress addr = new SnmpIpAddress(inet);
		vb = new SnmpVarBind(SnmpTrap.snmpTrapAddressOid,addr);
	    }catch(UnknownHostException e) {
		vb = new SnmpVarBind(SnmpTrap.snmpTrapAddressOid, 
				     new SnmpIpAddress("123.33.33.33")); 
	    }
	    lgarbage.addVarBind(vb);
	}
    }
    
    static SnmpAdaptorServer snmpAdaptor = null;

    /**
     * Main entry point.
     * When calling the program, you can specify:
     *
     * In order to get traces from the SNMP adaptor server, invoke the program as follows:
     *  java -DINFO_ADAPTOR_SNMP Agent
     */
    public static void main(String args[]) {
        
        MBeanServer server;
        ObjectName htmlObjName;
        ObjectName snmpObjName;
        ObjectName mibObjName;
        int htmlPort = 8092;
        int snmpPort = 8095;

	//Proxy args
	String proxyHost = "localhost";
	String proxyPort = "8085";
	if(args.length >= 1)
	    proxyHost = args[0];
	if(args.length >= 2)
	    proxyPort = args[1];


        // Initialize trace property.
        // 
        try {
            TraceManager.parseTraceProperties();
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", "Default trace is on");
        } catch(java.io.IOException e) {
            e.printStackTrace();
        }
    
        try {
            server = MBeanServerFactory.createMBeanServer();
            String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain + ":class=HtmlAdaptorServer,protocol=html,port=" + htmlPort);
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", 
                       "Adding HTML adaptor to MBean server with name \n\t" + htmlObjName);
            java.lang.System.out.println("NOTE: HTML adaptor is bound on TCP port " + htmlPort);
            HtmlAdaptorServer htmlAdaptor = new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();
                  
            //
            // SNMP specific code:
            //
      
            // Create and start the SNMP adaptor.
            // Specify the port to use in the constructor. 
            // If you want to use the standard port (161) comment out the following line:
            
            snmpPort = 8095;
            snmpObjName = new ObjectName(domain + ":class=SnmpAdaptorServer,protocol=snmp,port=" + snmpPort);
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", 
                       "Adding SNMP adaptor to MBean server with name \n\t" + snmpObjName);
            java.lang.System.out.println("NOTE: SNMP Adaptor is bound on UDP port " + snmpPort);
            snmpAdaptor = new SnmpAdaptorServer(snmpPort);
	    // The V3 proxy is listening on 8086.
	    snmpAdaptor.setTrapPort(new Integer(8086));
            server.registerMBean(snmpAdaptor, snmpObjName);
            snmpAdaptor.start();
            // Create an instance of the customized MIB
            //
            RFC1213_MIB mib2 = new RFC1213_MIB_IMPL();
      
            // Bind the SNMP adaptor to the MIB in order to make the MIB 
            // accessible through the SNMP protocol adaptor.
            // If this step is not performed, the MIB will still live in 
            // the JDMK agent:
            // its objects will be addressable through HTML but not SNMP.
            //
	    mib2.init();
	    snmpAdaptor.addMib(mib2);
	    
	    println("Press Return to register the agent in the proxy [" + proxyHost + ":"+proxyPort+"]");
	    java.lang.System.in.read();
	    
	    //Peer
	    SnmpPeer proxy = new SnmpPeer(proxyHost, 
					  Integer.parseInt(proxyPort));
	    SnmpParameters p = new SnmpParameters("public", "private");
	    SnmpSession session = new SnmpSession("Register session");
	    proxy.setParams(p);
	    session.setDefaultPeer(proxy);
	    
	    //Params registering
	    //agent_paramsV1,0,0,public,1,3
	    SnmpV3AppliMibRegistration.registerParams(session,
						      "agent_paramsV1",
						      0,
						      0,
						      "public",
						      1);
	    // The incoming matching criteria
	    // Only SMP V3 manager can acces this agent.
	    //agent_criteriaV3,3,3,defaultUser,2,3
	    SnmpV3AppliMibRegistration.registerParams(session,
						      "agent_criteriaV3",
						      3,
						      3,
						      "defaultUser",
						      2);
	    //Target registering
	    //agent_targetV1,snmpUDPDomain,localhost:8095,10000,2,useless list,agent_paramsV1,3
	    SnmpV3AppliMibRegistration.registerTarget(session,
						      "agent_targetV1",
						      "1.3.6.1.6.1.1",
						      "localhost/8095",
						      10000,
						      2,
						      "useless list",
						      "agent_paramsV1");
	    
	    //Proxy registering
	    //proxy_agent,1,localhost:8095,TEST-CONTEXT,snmpV3,agentV1,not used,3
	    SnmpV3AppliMibRegistration.registerProxy(session,
						     "proxy_agent",
						     1,
						     SnmpEngineId.createEngineId("localhost:8095"),
						     "TEST-CONTEXT",
						     "agent_criteriaV3",
						     "agent_targetV1",
						     "useless list");
	    
	    println("Agent successfuly registered in proxy");
	    println("Press Return to unregister this agent from the proxy");
	    java.lang.System.in.read();
	    SnmpV3AppliMibRegistration.unregisterParams(session,
							"agent_paramsV1");
	    SnmpV3AppliMibRegistration.unregisterParams(session,
							"agent_criteriaV3");
	    SnmpV3AppliMibRegistration.unregisterTarget(session,
							"agent_targetV1");
	    SnmpV3AppliMibRegistration.unregisterProxy(session,
						       "proxy_agent");

	    println("Agent successfuly unregistered from proxy");

	    println("Press Return if you want to send traps");
	    java.lang.System.in.read();
	    
	    //Instantiate and start the TrapGenerator.
	    TrapGenerator trapGenerator = new TrapGenerator(snmpAdaptor);
	    trapGenerator.start();

            java.lang.System.out.println(">> Press <Ctrl-C> if you want to stop this agent.");
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    static private void println(String msg) {
	java.lang.System.out.println(msg);
    }
}
