/*
 * @(#)file      InformListenerImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.io.IOException;

// RI import
//
import com.sun.management.snmp.SnmpPduRequest;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.manager.SnmpInformListener;

/**
 * This class implements the SnmpInformListener interface.
 * The callback method processSnmpInform is called when a valid inform 
 * PDU is received.
 */

public class InformListenerImpl implements SnmpInformListener {

    public void processSnmpInform(SnmpPduRequest inform) {
        
        // Display the received PDU.
        //
        println("\nNOTE: Inform request received:\n");
        println("\tType      = " + inform.pduTypeToString(inform.type));
        println("\tVersion   = " + inform.version);
        println("\tRequestId = " + inform.requestId);
        println("\tAddress   = " + inform.address);
        println("\tPort      = " + inform.port);
        println("\tCommunity = " + new String(inform.community));
        println("\tVB list   = ");
        for (int i = 0; i < inform.varBindList.length; i++) {
            println("\t\t" + inform.varBindList[i]);
        }
    }

    public void processSnmpInformV3(SnmpScopedPduRequest inform) {
	println("\nNOTE: Inform request V3 received:\n");
        println("\tType      = " + inform.pduTypeToString(inform.type));
        println("\tVersion   = " + inform.version);
        println("\tRequestId = " + inform.requestId);
        println("\tAddress   = " + inform.address);
        println("\tPort      = " + inform.port);
        println("\tContext = " + new String(inform.contextName));
        println("\tVB list   = ");
        for (int i = 0; i < inform.varBindList.length; i++) {
            println("\t\t" + inform.varBindList[i]);
        }
    }
    
    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}
