/*
 * @(#)file      Agent.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// JMX imports
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

// JDMK imports
//
import com.sun.management.snmp.SnmpPduPacket;
import com.sun.management.snmp.SnmpStatusException;

import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpAdaptorServer;

/**
 * The Agent class provides a simple example on how to use the SNMP
 * protocol adaptor.
 * It shows how to register a customized version (DEMO_MIB_IMPL) of
 * a mibgen generated mib (DEMO_MIB).
 *
 * You can now see the MIB using your favorite SNMP manager,
 * or you can use a web browser and see the MIB through the HTML adaptor.
 *
 * In this example, the SNMP adaptor is started on port 8085,
 * and the traps are sent to the port 8086, i.e non standard ports for SNMP.
 * You will thus not need to be root to start the agent.
 */

public class Agent {

    static SnmpAdaptorServer snmpAdaptor = null;

    /**
     * Main entry point.
     */
    public static void main(String args[]) {

        MBeanServer server;
        ObjectName htmlObjName;
        ObjectName snmpObjName;
        ObjectName mibObjName;
        ObjectName trapGeneratorObjName;
        int htmlPort = 8082;
        int snmpPort = 161;

        try {
            server = MBeanServerFactory.createMBeanServer();
            String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName =
		new ObjectName(domain +
			       ":class=HtmlAdaptorServer,protocol=html,port=" +
			       htmlPort);
            java.lang.System.out.println(
		      "Adding HTML adaptor to MBean server with name \n\t" +
                       htmlObjName);
            java.lang.System.out.println("NOTE: HTML adaptor is bound to " +
                                         "TCP port " + htmlPort);
            HtmlAdaptorServer htmlAdaptor = new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();

            //
            // SNMP specific code:
            //

            // Create and start the SNMP adaptor.
            // Specify the port to use in the constructor.
            // If you want to use the standard port (161)
	    // comment out the line:
            //
            // snmpPort = 8085;
            //
            snmpPort = 8085;
            snmpObjName =
		new ObjectName(domain +
			       ":class=SnmpAdaptorServer,protocol=snmp,port=" +
			       snmpPort);
            java.lang.System.out.println(
		      "Adding SNMP adaptor to MBean server with name \n\t" +
		      snmpObjName);
            java.lang.System.out.println("NOTE: SNMP Adaptor is bound to " +
                                         "UDP port " + snmpPort);
            snmpAdaptor = new SnmpAdaptorServer(snmpPort);
            server.registerMBean(snmpAdaptor, snmpObjName);

            // Start the SNMP Adaptor.
            //
            snmpAdaptor.start();

            // Create the DEMO-MIB and add it to the MBean server.
            //
            mibObjName= new ObjectName("snmp:class=DEMO_MIB");
            java.lang.System.out.println(
		      "Adding DEMO-MIB to MBean server with name \n\t" +
		      mibObjName);

            // Create an instance of the customized MIB
            //
            DEMO_MIB demo = new DEMO_MIB_IMPL();
            server.registerMBean(demo, mibObjName);

            // Bind the SNMP adaptor to the MIB in order to make the MIB
            // accessible through the SNMP protocol adaptor.
            // If this step is not performed, the MIB will still live in
            // the Java DMK agent:
            // its objects will be addressable through HTML but not SNMP.
            //
	    demo.setSnmpAdaptor(snmpAdaptor);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Return a reference to the SNMP adaptor server.
     */
    static public SnmpAdaptorServer getSnmpAdaptor() {
        return snmpAdaptor;
    }

    /**
     * Return usage of the program.
     */
    public static void  usage() {
        java.lang.System.out.println("java Agent");
    }
}
