/*
 * @(#)file      DEMO_MIB_IMPL.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.3
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;
import java.util.Hashtable;

// jmx imports
//
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.InstanceAlreadyExistsException;

// jdmk imports
//
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibNode;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpStandardObjectServer;

/**
 * This class implements a customized "DEMO-MIB".
 * We simply redefine the factory method for the "Demo" group MBean
 * in order to instantiate our own customized class.
 */
public class DEMO_MIB_IMPL extends DEMO_MIB {

    /**
     * Default constructor. Initialize the Mib tree.
     */
    public DEMO_MIB_IMPL() {
        super();
    }

    /**
     * Factory method for "Demo" group MBean.
     * 
     * We redefine this method in order to replace the default
     * generated MBean class with our own customized class.
     * 
     * @param groupName Name of the group ("Demo")
     * @param groupOid  OID of this group
     * @param groupObjname ObjectName for this group (may be null)
     * @param server    MBeanServer for this group (may be null)
     * 
     * @return An instance of the customized MBean class for the
     *         "Demo" group (DemoImpl)
     * 
     **/
    protected Object createDemoMBean(String groupName,
                String groupOid,  ObjectName groupObjname, 
		MBeanServer server)  {

        if (server != null) 

	    // We instantiate our own DemoImpl class instead of the
	    // default Demo class.
	    //
            return new DemoImpl(this,server);

        else 

	    // We instantiate our own DemoImpl class instead of the
	    // default Demo class.
	    //
            return new DemoImpl(this);
    }


}
